//
// TriggerWatchdog
//
// WatchdogTrigger_EP9315
// ADS 720020-1195x
//
// Sample programming for testing the WatchDog Timer on the EP9315 CPU
//
// Uses the watchdog timer to trigger a hardware reset of the system.
// The program will run one of two tests based on the contents of the
// WatchDog status register.
//
// If the status register is equal to RESET_TEST (all 0's, usually resulting 
// from power on, or a run of the Disable Test), the status register will be
// set to DISABLE_TEST (for the next program execution after reset), and a reset
// test will be run, where the WatchDog is enabled and the program is
// put to sleep in 1 ms increments and then the WatchDog is petted, until the 
// sleep is too long for a Pet() to prevent a system reset.
//
// Upon system restart, if the program is run again, the value found in the 
// status register will indicated a Disable Test is to be run, and the value
// of RESET_TEST will be stored in the status register (for the next program 
// execution), and the Disable Test will be run, where the WatchDog is enabled, 
// petted once, and then disabled, resulting in no system restart.
//
// This test cycle will continue each time the test program is run.
//
// jn 15 Aug 2006 

#include "stdafx.h"
#include "watchdog.h"

#define RESET_TEST 1
#define DISABLE_TEST 2

#define RESET_STATUS    0x00000000
#define DISABLE_STATUS  0x00000055

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	int test_type;
	
	RETAILMSG(1,(L"\r\nTrigger Watchdog Test:  ADS 720020-1195x\r\n\r\n"));

	// Create a WatchDog timer
	WatchDog *wd = new WatchDog(1);

	// Read the contents of the status register to determine the test to run
	ULONG status = wd->GetStatus();

	RETAILMSG(1,(L"Contents of Status Register:  0x%x\r\n", status));

	if(status == RESET_STATUS)
	{
		test_type = RESET_TEST;
		status = DISABLE_STATUS;
		RETAILMSG(1,(L"Reset Test...\r\n"));
	}
	else 
	{
		test_type = DISABLE_TEST;
		status = RESET_STATUS;
		RETAILMSG(1,(L"Disable Test...\r\n"));
	}
	
	// Set the status register so next time program is run, the other test type is run
	RETAILMSG(1,(L"Setting Status Register:  0x%x\r\n\r\n", status));
	wd->SetStatus(status);
	
	// Test IsEnabled Functionallity
	if(wd->IsEnabled())
		RETAILMSG(1,(L"Watchdog already enabled\r\n\r\n"));
	else
	{
		RETAILMSG(1,(L"Enabling Watchdog...\r\n"));
		wd->Enable();

		if(wd->IsEnabled())
		{
			RETAILMSG(1,(L"Watchdog is enabled\r\n\r\n"));

			// An Initial pet
			wd->Pet();

			// Disable Test
			if(test_type == DISABLE_TEST)
			{
				wd->Disable();
				RETAILMSG(1,(L"... Watchdog Disabled\r\n"));
			}
			// Reset Test, cause the watchdog to reset the system
			else if(test_type == RESET_TEST)
			{
				// Progressively sleep the system until a sleep is too long for the watchdog to be petted on time
				for(int i = 240; i < 260; i++)
				{	
					RETAILMSG(1,(L"Sleep %d...\r\n", i));
					Sleep(i);
					wd->Pet();
				}
			}
		}
		else
		{
			RETAILMSG(1,(L"ERROR:  Watchdog cannot be enabled\r\n"));
		}
	}

	RETAILMSG(1,(L"Test Finished\r\n"));
	return 0;
}

